-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

-- Possible generic(s) values: 
--		VERSION = {SHA3_ROUND2, SHA3_ROUND3}
--		HS = {HASH_SIZE_256, HASH_SIZE_512}
--		W = {32, 64}

-- Valid combinations
-- 		(HS, W) = (HASH_SIZE_256, 32), (HASH_SIZE_512, 64)


library ieee;
use ieee.std_logic_1164.all; 
use work.sha3_pkg.all;
use work.sha3_blake_package.all;

entity blake_top is		
	generic (		
		VERSION : integer := SHA3_ROUND3;		
		HS : integer := HASH_SIZE_256;
		w : integer := 32
	); 
	port (		
		-- global
		rst 	: in std_logic;
		clk 	: in std_logic;	
		--fifo
		src_ready : in std_logic;
		src_read  : out std_logic;
		dst_ready : in std_logic;
		dst_write : out std_logic;		
		din		: in std_logic_vector(w-1 downto 0);
		dout	: out std_logic_vector(w-1 downto 0)
	);	   
end blake_top;


architecture structure of blake_top is  
	constant b : integer := get_b( HS );

	signal msg_wr, state_wr, h_wr : std_logic;
	signal sf, load_c, ec, eth, etl, dt : std_logic;
	signal stbot, zc0, zcRemainder, extra_block, smux : std_logic;
	signal round : std_logic_vector(7 downto 0);
begin			   
	control_gen : entity work.blake_control(struct) 
		generic map ( VERSION => VERSION, HS => HS )
		port map (
		rst => rst, clk => clk, 
		src_ready => src_ready, src_read => src_read, dst_ready => dst_ready, dst_write => dst_write,	  
		final_segment => din(w-1),
		round => round, msg_wr => msg_wr, state_wr => state_wr, h_wr => h_wr,
		sf => sf, load_c => load_c, ec => ec, eth => eth, etl => etl, dt => dt, smux => smux,
		stbot => stbot, zc0 => zc0, zcRemainder => zcRemainder, extra_block => extra_block
	);			
	
	datapath_gen : entity work.blake_datapath(struct) 	
		generic map ( W => W, HS => HS )
		port map (
		clk => clk, din => din, dout => dout, 
		round => round, msg_wr => msg_wr, state_wr => state_wr, h_wr => h_wr,
		sf => sf, load_c => load_c, ec => ec, eth => eth, etl => etl, dt => dt, smux => smux,
		stbot => stbot, zc0 => zc0, zcRemainder => zcRemainder, extra_block => extra_block
	);

end structure;
	
	